//This class, Enemy2, is written by Joseph Miller on 10-19-09
using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace SuperScooter
{
    /// <summary>
    /// This is a game component that implements the second enemy class.
    /// </summary>
    public class Enemy2 : Collideable   //Microsoft.Xna.Framework.DrawableGameComponent 
    {
        GraphicsDevice graphics;
        Game _game;

        #region Constants
        public const double GRAVITY = 0.09;
        //Added by Rae Melton----------
        public const int Y_DISPLACEMENT_FOR_TILE_GRAPHICS = 5;
        public const int HORIZONTAL_FRAME_GAP = 10;
        public const float SPRITE_DEFAULT_ALPHA = 1.0f;
        public const float SPRITE_ALPHA_CHANGE = 0.05f;
        //-----------------------------
        #endregion

        #region Private Members

        //Added by Rae Melton----------
        private Sprite enemySprite;
        private Texture2D spriteTexture;
        private Vector2 _dyingPosition;
        //-----------------------------
        private Vector2 _spritePosition;
        private Vector2 _spriteSpeed = new Vector2(0.5f,0);
        private Vector2 _spriteBounce = new Vector2(0, (float)GRAVITY);
        private Vector2 startOfBounce;

        #endregion

        #region Flags

        private bool lastBounce = false;
        private bool dying = false;
        private bool dead = false;
        private bool visible = true;
        private bool enabled = true;
        private bool bounceUpDown = false;
        private bool bounceLeftRight = false;

        #endregion

        #region Public Properties

        public Texture2D Texture
        {
            get { return Avatar; }
            set { Avatar = value; }
        }
        public Vector2 Position
        {
            get { return _spritePosition; }
        }
        public Vector2 Speed
        {
            get { return _spriteSpeed; }
        }
        public Vector2 Bounce
        {
            get { return _spriteBounce; }
        }
        public Rectangle Shell
        {
            get { return shell; }
        }
        public bool Visible
        {
            get { return visible; }
        }
        public bool Enabled
        {
            get { return enabled; } 
        }
        public bool Dead
        {
            get { return dead; }
        }
        #endregion

        #region Tile Members
        //tile stuff
        private List<List<Tile>> Tiles;
        Tile currentBottom = new Tile(null, CollideType.Passable, 0, 0);
        Tile currentTop = new Tile(null, CollideType.Passable, 0, 0);
        Tile left1 = new Tile(null, CollideType.Passable, 0, 0);
        Tile right1 = new Tile(null, CollideType.Passable, 0, 0);
        Tile left2 = new Tile(null, CollideType.Passable, 0, 0);
        Tile right2 = new Tile(null, CollideType.Passable, 0, 0);
        Tile top = new Tile(null, CollideType.Passable, 0, 0);
        Tile top_right = new Tile(null, CollideType.Passable, 0, 0);
        Tile top_left = new Tile(null, CollideType.Passable, 0, 0);
        Tile bottom = new Tile(null, CollideType.Passable, 0, 0);
        Tile bottom_right = new Tile(null, CollideType.Passable, 0, 0);
        Tile bottom_left = new Tile(null, CollideType.Passable, 0, 0);
        #endregion

        #region Constructor
        /// <summary>
        /// The constructor will need the game object, it's initial position, and a reference to the Tile array
        /// </summary>
        /// <param name="game">The game object.</param>
        /// <param name="pos">The initial position</param>
        /// <param name="tiles">reference to Tile array</param>
        public Enemy2(Game game, Vector2 pos, ref List<List<Tile>> tiles)
        {
            _game = game;
            _spritePosition = pos;
            this.Tiles = tiles;
            LoadContent();
            Initialize();
        }

        #endregion

        #region Load / Unload Content
        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        public void LoadContent()
        {
            Texture = _game.Content.Load<Texture2D>(@"textures\crane_still");

            //Added by Rae Melton----------
            spriteTexture = _game.Content.Load<Texture2D>(@"textures\crane");
            enemySprite = new Sprite(_game, spriteTexture, Sprite.Direction.LEFT, Sprite.Direction.LEFT, Tile.SIZE, 2 * Tile.SIZE, HORIZONTAL_FRAME_GAP, 7, 6, -1);
            enemySprite.ChangeDirection();//this is because the enemy starts heading right for some reason >.>
            //-----------------------------
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public void Initialize()
        {            
            //Initialize the graphics manager
            graphics = _game.GraphicsDevice;

            //set up the shell
            SetPosition(_spritePosition);

            //set up the speed for the enemy
            _spriteSpeed = new Vector2(1f, 0);

            //initially assign startOfBounce to beginning position
            startOfBounce = _spritePosition;
        }

        #endregion

        #region Update

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public void Update(GameTime gameTime)
        {
            if (Enabled)
            {
                //if dead, stop moving and slowly shrink to nothing
                if (dying)
                {
                    //Changed by Rae Melton---------------------
                    //slowly fade the enemy out
                    if (enemySprite.AlphaValue > 0)
                        enemySprite.AlphaValue -= SPRITE_ALPHA_CHANGE;
                    else
                        visible = enabled = dying = !(dead = true);
                    //-------------------------------------------
                }
                else
                {
                    //if(enemy has bounced above 3 tiles) set speed to 0
                    if ((startOfBounce.Y - _spritePosition.Y) > (Tile.SIZE * 3))
                    {
                        _spriteSpeed.Y = 0;
                    }
                    if (_spriteSpeed.Y >= 0)//return bounce to normal
                    {
                        _spriteBounce.Y = (float)GRAVITY;
                    }

                    enemySprite.ChangeState();
                    //update the tiles around you
                    try
                    {
                        int tilePositionX = (int)_spritePosition.X / 40;
                        int tilePositionY = (int)_spritePosition.Y / 40;
                        currentTop = Tiles[tilePositionX][tilePositionY];
                        currentBottom = Tiles[tilePositionX][tilePositionY + 1];
                        top = Tiles[tilePositionX][tilePositionY - 1];
                        top_right = Tiles[tilePositionX + 1][tilePositionY - 1];
                        top_left = Tiles[tilePositionX - 1][tilePositionY - 1];
                        right1 = Tiles[tilePositionX + 1][tilePositionY];
                        left1 = Tiles[tilePositionX - 1][tilePositionY];
                        right2 = Tiles[tilePositionX + 1][tilePositionY + 1];
                        left2 = Tiles[tilePositionX - 1][tilePositionY + 1];
                        bottom = Tiles[tilePositionX][tilePositionY + 2];
                        bottom_right = Tiles[tilePositionX + 1][tilePositionY + 2];
                        bottom_left = Tiles[tilePositionX - 1][tilePositionY + 2];
                    }
                    catch (Exception e)
                    {
                        string trash = e.Message;
                        //dying = true;
                    }

                    if (currentTop.collideType == CollideType.Impassable || currentBottom.collideType == CollideType.Impassable)
                    {
                        bounceLeftRight = true;
                        bounceUpDown = true;
                    }

                    //check for collisions
                    int collideCounter = 0;
                    if (CheckCollide(left1, false) || CheckCollide(left2, false) || CheckCollide(top_left, false) || CheckCollide(bottom_left, false))
                    {
                        while (CheckCollide(left1, false) || CheckCollide(left2, false) || CheckCollide(top_left, false) || CheckCollide(bottom_left, false))
                        {
                            _spritePosition.X++;
                            SetPosition(_spritePosition);
                        }
                        bounceLeftRight = true;
                    }
                    if (CheckCollide(bottom, true) || CheckCollide(bottom_right, true) )//|| CheckCollide(bottom_left, true))
                    {
                        while (CheckCollide(bottom, true) || CheckCollide(bottom_right, true) && collideCounter < 10)//|| CheckCollide(bottom_left, true))
                        {
                            _spritePosition.Y--;
                            SetPosition(_spritePosition);
                            collideCounter++;
                        } if (shell.Intersects(Tiles[(int)_spritePosition.X / 40][(int)_spritePosition.Y / 40 - 1].BoundingRectangle))
                            MoveOutOfWay(new Vector2((_spriteSpeed.X)*Tile.SIZE,0));
                        bounceUpDown = true;
                    }
                    if (CheckCollide(right1, false) || CheckCollide(right2, false) || CheckCollide(top_right, false) || CheckCollide(bottom_right, false))
                    {
                        while (CheckCollide(right1, false) || CheckCollide(right2, false) || CheckCollide(top_right, false) || CheckCollide(bottom_right, false))
                        {
                            _spritePosition.X--;
                            SetPosition(_spritePosition);
                        }
                        bounceLeftRight = true;
                    }
                    if (CheckCollide(top, false) || CheckCollide(top_right, false) || CheckCollide(top_left, false))
                    {
                        while (CheckCollide(top, false) || CheckCollide(top_right, false) || CheckCollide(top_left, false))
                        {
                            _spritePosition.Y++;
                            SetPosition(_spritePosition);
                        }
                        bounceUpDown = true;
                    }

                    if (bounceUpDown)
                    {
                        BounceOnVertical();
                        bounceUpDown = false;
                    }
                    if (bounceLeftRight)
                    {
                        BounceOnHorizontal();
                        enemySprite.ChangeDirection();
                        bounceLeftRight = false;
                    }

                    if (!dying && !dead)
                    {
                        //add gravity to the enemy
                        _spriteSpeed += Bounce;


                        //move the enemy
                        _spritePosition += Speed;

                        //adjust the position
                        SetPosition(_spritePosition);
                    }
                }
            }
        }
        #endregion

        #region Auxilary Functions
        private bool CheckCollide(Tile tile, bool isBottom)
        {
            return shell.Intersects(tile.BoundingRectangle) && (tile.collideType == CollideType.Impassable || ((tile.collideType == CollideType.Platform || tile.collideType == CollideType.Enemy2Platform) && isBottom));
        }
        /// <summary>
        /// Makes the enemy bounce upwards or downwards, depending on which way it was originally traveling.
        /// </summary>
        public void BounceOnVertical()
        {
            if (!dying)
            {
                //if the sprite was heading downward
                //reset clamp on max height of jump
                if (_spriteSpeed.Y >= 0)
                    startOfBounce = _spritePosition;

                if (lastBounce)
                {
                    _spriteBounce *= 0;
                    _spriteSpeed.Y = 0;
                }
                else
                    _spriteBounce.Y = (float)GRAVITY;

                if (_spriteSpeed.Y == 0)
                    _spriteSpeed.Y = 1;
                else
                    _spriteSpeed.Y *= -1;
                _spriteBounce.Y += 0.005f;
            }
        }

        /// <summary>
        /// Makes the enemy bounce to the left or to the right, depending on which way it was originally traveling.
        /// </summary>
        public void BounceOnHorizontal()
        {
            _spriteSpeed.X *= -1;
            _spriteBounce.Y += 0.001f;
        }

        /// <summary>
        /// This function will move an enemy's position by the supplied amount
        /// </summary>
        /// <param name="v">A vector of how far and in which direction to move it</param>
        public void MoveOutOfWay(Vector2 v)
        {
            _spritePosition += v;
        }

        public void TakeHit()
        {
            //if you hit it once, stop the bouncing
            if (lastBounce == false)
            {
                lastBounce = true;
                _spriteSpeed = new Vector2(_spriteSpeed.X, 0);
            }
            else //otherwise the enemy dies
            {
                dying = true;

                //make the enemy no longer collideable
                _dyingPosition = new Vector2(shell.Left, shell.Top);
                shell = new Rectangle(0, 0, 0, 0);

                //change the sprite frame to the death state
                enemySprite.KillSprite();

                //make the enemy stop moving
                _spriteSpeed = new Vector2(0, 0);
            }
        }
        #endregion

        #region Draw

        /// <summary>
        /// This is called when the game needs to draw Enemy2.
        /// </summary>
        /// <param name="spriteBatch">SpriteBatch used to draw the enemy's texture to the screen.</param>
        /// <param name="CameraPosition">Camera offset needed to accurately draw the enemy to the screen.</param>
        public void Draw(SpriteBatch spriteBatch, Point CameraPosition)
        {
            if (dying)
                enemySprite.Draw(spriteBatch, (int)(_dyingPosition.X - CameraPosition.X), (int)_dyingPosition.Y - CameraPosition.Y + Y_DISPLACEMENT_FOR_TILE_GRAPHICS);
            else
                enemySprite.Draw(spriteBatch, (int)(shell.X - CameraPosition.X), (int)shell.Y - CameraPosition.Y + Y_DISPLACEMENT_FOR_TILE_GRAPHICS);
        }

        #endregion
    }
}