using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;
using System.IO;


namespace SuperScooter
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class Level : IDisposable
    {
        private const float DARKX_ALPHA_CHANGE = 0.005f;
        private const float DARKX_ALPHA_MAX = 0.5f;

        ContentManager content;
        private List<List<Tile>> tiles;
        private List<List<Tile>> midground;
        private int midgroundWidth = 0;
        private List<List<Tile>> background;
        private int backgroundWidth = 0;
        private Texture2D stillBackground;

        private bool cheatOn = false;
        private bool enemiesOn = true;

        private SpriteFont font;
        
        int numRows = 0;
        int numCols = 0;
        int numBackRows = 0;
        int numBackCols = 0;
        int numMidRows = 0;
        int numMidCols = 0;
        int levelNumber;
        Point playerStart;
        //added by Joseph Miller
        //Added by Rae Melton-
        private float nightShadeAlpha = 0.0f;
        private Texture2D nightShadeTexture;
        //--------------------
        int nightShadeStart = -1;
        //----------------------

        private Point finishLine;
        public Point FinishLine
        {
            get { return finishLine; }
        }

        Point levelSize;
        public Point LevelSize
        {
            get { return levelSize; }
        }

        Point cameraPosition;
        public Point CameraPosition
        {
            get { return cameraPosition; }
        }

        int CameraVerticalLock = 0;
        public Level(Game game, int levelNumber)
        {
            content = new ContentManager(game.Services, @"Content\Textures\Level" + levelNumber);
            this.levelNumber = levelNumber;
            cameraPosition.X = 0;
            
        }

        public List<List<Tile>> Tiles
        {
            get { return tiles; }
        }

        public void LoadContent(Game game, out Enemy1[] enemies, out Enemy2[] enemies2, ref player player1, string playerFileName)
        {
            string line;

            const int MAX_ENEMIES = 100;
            enemies = new Enemy1[MAX_ENEMIES];
            enemies2 = new Enemy2[MAX_ENEMIES];

            int enemyCount = 0;
            int enemyCount2 = 0;

            //Foreground Textures
            //Added by Rae Melton------------------------------------------------------
            Texture2D groundTopLeft = content.Load<Texture2D>("ground_top_left");
            Texture2D groundTopCenter = content.Load<Texture2D>("ground_top_center");
            Texture2D groundTopRight = content.Load<Texture2D>("ground_top_right");
            Texture2D groundBottomLeft = content.Load<Texture2D>("ground_bottom_left");
            Texture2D groundBottomCenter = content.Load<Texture2D>("ground_bottom_center");
            Texture2D groundBottomRight = content.Load<Texture2D>("ground_bottom_right");
            Texture2D platformLeft = content.Load<Texture2D>("platform_left");
            Texture2D platformCenter = content.Load<Texture2D>("platform_center");
            Texture2D platformRight = content.Load<Texture2D>("platform_right");
            Texture2D invertedCornerLeft = content.Load<Texture2D>("ground_inverted_left");
            Texture2D invertedCornerRight = content.Load<Texture2D>("ground_inverted_right");
            Texture2D treeTrunk1Left = content.Load<Texture2D>("trunkLeft");
            Texture2D treeTrunk1Right = content.Load<Texture2D>("trunkRight");
            Texture2D treeTrunkSplitLeft = content.Load<Texture2D>("trunkSplitleft");
            Texture2D treeTrunkSplitRight = content.Load<Texture2D>("trunkSplitright");
            Texture2D treePlatformLeft = content.Load<Texture2D>("tree_platform_left");
            Texture2D treePlatform = content.Load<Texture2D>("tree_platform");
            Texture2D treePlatformRight = content.Load<Texture2D>("tree_platform_right");
            Texture2D empty = content.Load<Texture2D>("empty");
            Texture2D acorn = content.Load<Texture2D>("acorn");
            Texture2D rock = content.Load<Texture2D>("rock");
            Texture2D tree = content.Load<Texture2D>("tree");
            Texture2D ceiling = content.Load<Texture2D>("ceiling_tile");

            Texture2D[] bgImages = new Texture2D[5];
            bgImages[0] = content.Load<Texture2D>("background1");
            bgImages[1] = content.Load<Texture2D>("background2");
            bgImages[2] = content.Load<Texture2D>("background3");
            bgImages[3] = content.Load<Texture2D>("background4");
            bgImages[4] = content.Load<Texture2D>("background5");
            backgroundWidth = bgImages[0].Width;

            Texture2D[] mgImages = new Texture2D[4];
            mgImages[0] = content.Load<Texture2D>("midground1");
            mgImages[1] = content.Load<Texture2D>("midground2");
            mgImages[2] = content.Load<Texture2D>("midground3");
            mgImages[3] = content.Load<Texture2D>("midground4");
            midgroundWidth = mgImages[0].Width;

            nightShadeTexture = content.Load<Texture2D>("night_shade");
            stillBackground = content.Load<Texture2D>("bg");

            ContentManager temp = new ContentManager(game.Services, @"Content/Fonts/");
            font = temp.Load<SpriteFont>("happy36");
            //-------------------------------------------------------------------------

            string fileName = "level" + levelNumber + ".txt";
            string backgroundFileName = "level" + levelNumber + "Background.txt";
            string midgroundFileName = "level" + levelNumber + "Midground.txt";

            StreamReader levelFile = new StreamReader(@"Content/Levels/" + fileName);
            StreamReader backgroundFile = new StreamReader(@"Content/Levels/" + backgroundFileName);
            StreamReader midgroundFile = new StreamReader(@"Content/Levels/" + midgroundFileName);

            #region Tile List Population
            numRows = 0;
            numCols = 0;
            tiles = new List<List<Tile>>();
            int i = 0;
            do
            {
                line = levelFile.ReadLine();
                numCols = line.Length;
                numRows++;

                for (int j = 0; j < numCols; j++)
                {
                    tiles.Add(new List<Tile>());
                    switch (line[j])
                    {
                        //Added by Rae Melton------------------------------------------------------
                        case '+':
                            tiles[j].Add(new Tile(groundTopCenter, CollideType.Impassable, j, i));
                            break;
                        case ')':
                            tiles[j].Add(new Tile(groundTopRight, CollideType.Impassable, j, i));
                            break;
                        case '(':
                            tiles[j].Add(new Tile(groundTopLeft, CollideType.Impassable, j, i));
                            break;
                        case '=':
                            tiles[j].Add(new Tile(groundBottomCenter, CollideType.Impassable, j, i));
                            break;
                        case ']':
                            tiles[j].Add(new Tile(groundBottomRight, CollideType.Impassable, j, i));
                            break;
                        case '[':
                            tiles[j].Add(new Tile(groundBottomLeft, CollideType.Impassable, j, i));
                            break;
                        case '-':
                            tiles[j].Add(new Tile(platformCenter, CollideType.Platform, j, i));
                            break;
                        case '~':
                            if (cheatOn)
                                tiles[j].Add(new Tile(ceiling, CollideType.Impassable, j, i));
                            else
                                tiles[j].Add(new Tile(ceiling, CollideType.Passable, j, i));
                            break;
                        case '>':
                            tiles[j].Add(new Tile(platformRight, CollideType.Platform, j, i));
                            break;
                        case '<':
                            tiles[j].Add(new Tile(platformLeft, CollideType.Platform, j, i));
                            break;
                        case 'R':
                            tiles[j].Add(new Tile(rock, CollideType.Impassable, j, i));
                            break;
                        case 'r':
                            tiles[j].Add(new Tile(rock, CollideType.Platform, j, i));
                            break;
                        case 'T':
                            tiles[j].Add(new Tile(tree, CollideType.Impassable, j, i));
                            break;
                        case 'L':
                            tiles[j].Add(new Tile(invertedCornerRight, CollideType.Impassable, j, i));
                            break;
                        case 'J':
                            tiles[j].Add(new Tile(invertedCornerLeft, CollideType.Impassable, j, i));
                            break;
                        case 'u':
                            tiles[j].Add(new Tile(treeTrunk1Left, CollideType.Platform, j, i, new Rectangle(Tile.SIZE / 2, 0, Tile.SIZE, Tile.SIZE)));
                            break;
                        case 'n':
                            tiles[j].Add(new Tile(treeTrunk1Right, CollideType.Passable, j, i));
                            break;
                        case 'U':
                            tiles[j].Add(new Tile(treeTrunk1Left, CollideType.Passable, j, i));
                            break;
                        case 'N':
                            tiles[j].Add(new Tile(treeTrunk1Right, CollideType.Passable, j, i));
                            break;
                        case 'V':
                            tiles[j].Add(new Tile(treeTrunkSplitLeft, CollideType.Platform, j, i));
                            break;
                        case 'M':
                            tiles[j].Add(new Tile(treeTrunkSplitRight, CollideType.Platform, j, i));
                            break;
                        case '{':
                            tiles[j].Add(new Tile(treePlatformLeft, CollideType.Platform, j, i));
                            break;
                        case '_':
                            tiles[j].Add(new Tile(treePlatform, CollideType.Platform, j, i));
                            break;
                        case '}':
                            tiles[j].Add(new Tile(treePlatformRight, CollideType.Platform, j, i));
                            break;
                        case '*':
                            tiles[j].Add(new Tile(null, CollideType.Enemy2Platform, j, i));
                            break;
                        //-------------------------------------------------------------------------
                        //Changed by Rae Melton
                        case 'b':
                            tiles[j].Add(new Tile(null, CollideType.Boundary, j, i));
                            break;
                        case 'B':
                            tiles[j].Add(new Tile(null, CollideType.Impassable, j, i));
                            break;
                        //below added by Joseph Miller
                        case 'd':
                            tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                            nightShadeStart = j;
                            break;
                        //---------------------
                        case '.':
                            tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                        //Changed by Rae Melton
                        case 'X':
                            tiles[j].Add(new Tile(acorn, CollideType.Passable, j, i));
                            finishLine = tiles[j][i].BoundingRectangle.Center;
                            break;
                        //---------------------
                        case 'p':
                            tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                            player1 = new player(game, new Vector2(j * 40, i * 40), playerFileName, ref tiles);
                            playerStart = new Point(j,i);
                            break;
                        case '1':
                            if (enemiesOn)
                            {
                                tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                                enemies[enemyCount] = new Enemy1(game, new Vector2(j * 40, i * 40), ref tiles);
                                enemyCount++;
                            }
                            else
                                tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                        case '2':
                            if (enemiesOn)
                            {
                                tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                                enemies2[enemyCount2] = new Enemy2(game, new Vector2(j * 40, i * 40), ref tiles);
                                enemyCount2++;
                            }
                            else
                                tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                        case '^':
                            tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                            CameraVerticalLock = i;
                            break;
                        default:
                            tiles[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                    }
                }
                i++;
            } while (!levelFile.EndOfStream);

            cameraPosition.X = (int)MathHelper.Clamp((float)playerStart.X, 3.0f, (float)numCols + 3) * Tile.SIZE;
            cameraPosition.Y = (int)MathHelper.Clamp(numRows - SizeInTiles(Game1.CLIENT_HEIGHT), CameraVerticalLock, numRows) * Tile.SIZE;
            nightShadeStart = (nightShadeStart == -1) ? numCols : nightShadeStart;

            levelSize = new Point(numCols * Tile.SIZE - 1, numRows * Tile.SIZE - 1);
        #endregion

            #region Bacjground List Population
            numBackRows = 0;
            numBackCols = 0;
            background = new List<List<Tile>>();
            i = 0;
            do
            {
                line = backgroundFile.ReadLine();
                numBackCols = line.Length;
                numBackRows++;

                for (int j = 0; j < numBackCols; j++)
                {
                    background.Add(new List<Tile>());
                    switch (line[j])
                    {
                        case '1':
                            background[j].Add(new Tile(bgImages[0], CollideType.Passable, j, i));
                            break;
                        case '2':
                            background[j].Add(new Tile(bgImages[1], CollideType.Passable, j, i));
                            break;
                        case '3':
                            background[j].Add(new Tile(bgImages[2], CollideType.Passable, j, i));
                            break;
                        case '4':
                            background[j].Add(new Tile(bgImages[3], CollideType.Passable, j, i));
                            break;
                        case '.':
                            background[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                        default:
                            background[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                    }
                }
            } while (!backgroundFile.EndOfStream);
            #endregion

            #region Midground List Population
            numMidRows = 0;
            numMidCols = 0;
            midground = new List<List<Tile>>();
            i = 0;
            do
            {
                line = midgroundFile.ReadLine();
                numMidCols = line.Length;
                numMidRows++;

                for (int j = 0; j < numMidCols; j++)
                {
                    midground.Add(new List<Tile>());
                    switch (line[j])
                    {
                        case '1':
                            midground[j].Add(new Tile(mgImages[0], CollideType.Passable, j, i));
                            break;
                        case '2':
                            midground[j].Add(new Tile(mgImages[1], CollideType.Passable, j, i));
                            break;
                        case '3':
                            midground[j].Add(new Tile(mgImages[2], CollideType.Passable, j, i));
                            break;
                        case '4':
                            midground[j].Add(new Tile(mgImages[3], CollideType.Passable, j, i));
                            break;
                        case '5':
                            midground[j].Add(new Tile(mgImages[4], CollideType.Passable, j, i));
                            break;
                        case '.':
                            midground[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                        default:
                            midground[j].Add(new Tile(null, CollideType.Passable, j, i));
                            break;
                    }
                }
            } while (!midgroundFile.EndOfStream);
            #endregion
        }


        public void MoveCamera(int shift)
        {
            if (cameraPosition.X + shift > Tile.SIZE * 2 && cameraPosition.X + shift < LevelSize.X - Tile.SIZE * 2 - Game1.CLIENT_WIDTH)
            {
                cameraPosition.X += shift;
            }
        }

        public void MoveCameraVertically(int shift)
        {
            cameraPosition.Y += shift;

            if (!(CameraPosition.Y >= CameraVerticalLock * Tile.SIZE && cameraPosition.Y <= (numRows - SizeInTiles(Game1.CLIENT_HEIGHT)) * Tile.SIZE))
            {
                cameraPosition.Y -= shift;
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {

            int leftBackBlock = (cameraPosition.X / 3 - (cameraPosition.X / 3) % Tile.SIZE) / Tile.SIZE - SizeInTiles(backgroundWidth);
            int rightBackBlock = leftBackBlock + 2 * SizeInTiles(backgroundWidth) + SizeInTiles(Game1.CLIENT_WIDTH);

            int leftMidBlock = (cameraPosition.X / 2 - cameraPosition.X / 2 % Tile.SIZE) / Tile.SIZE - SizeInTiles(midgroundWidth);
            int rightMidBlock = leftMidBlock + 2 * SizeInTiles(midgroundWidth) + SizeInTiles(Game1.CLIENT_WIDTH);

            int leftBlock = (cameraPosition.X - cameraPosition.X % Tile.SIZE) / Tile.SIZE - SizeInTiles(Tile.SIZE);
            int rightBlock = leftBlock + 2 * SizeInTiles(Tile.SIZE) + SizeInTiles(Game1.CLIENT_WIDTH);

            if (leftBackBlock < 0)
                leftBackBlock = 0;
            if (leftMidBlock < 0)
                leftMidBlock = 0;
            if (leftBlock < 0)
                leftBlock = 0;

            if (rightBackBlock > numBackCols)
                rightBackBlock = numBackCols;
            if (rightMidBlock > numMidCols)
                rightMidBlock = numMidCols;
            if (rightBlock > numCols)
                rightBlock = numCols;

            //spriteBatch.Draw(stillBackground, new Vector2(0, 0), Color.White);

            for (int i = leftBackBlock; i < rightBackBlock; i++)
                for (int j = 0; j < numBackRows; j++)
                {
                    if (background[i][j].Avatar != null)
                        spriteBatch.Draw(background[i][j].Avatar, new Vector2(i * Tile.SIZE - cameraPosition.X / 3, j * Tile.SIZE - cameraPosition.Y), Color.White);
                }
            for (int i = leftMidBlock; i < rightMidBlock; i++)
                for (int j = 0; j < numMidRows; j++)
                {
                    if (midground[i][j].Avatar != null)
                        spriteBatch.Draw(midground[i][j].Avatar, new Vector2(i * Tile.SIZE - cameraPosition.X / 2, j * Tile.SIZE - cameraPosition.Y), Color.White);
                }
            for (int i = leftBlock; i < rightBlock; i++)
                for (int j = 0; j < numRows; j++)
                {
                    if (tiles[i][j].Avatar != null)
                        spriteBatch.Draw(tiles[i][j].Avatar, new Vector2(i * Tile.SIZE - cameraPosition.X, j * Tile.SIZE - cameraPosition.Y), Color.White);
                }

            if (nightShadeAlpha < DARKX_ALPHA_MAX && cameraPosition.X / Tile.SIZE > nightShadeStart)
                nightShadeAlpha += DARKX_ALPHA_CHANGE;
            spriteBatch.Draw(nightShadeTexture, new Vector2(0,0), new Color(Color.White, nightShadeAlpha));

            if (cheatOn)
                spriteBatch.DrawString(font, ">8D", new Vector2(10, Game1.CLIENT_HEIGHT - Tile.SIZE), Color.Red);
            if (!enemiesOn)
                spriteBatch.DrawString(font, "o,.,o", new Vector2(10, Game1.CLIENT_HEIGHT - 2*Tile.SIZE), Color.Red);
        }

        public void Dispose()
        {
            content.Unload();
        }

        public void setCheat(bool on)
        {
            if (on)
                cheatOn = true;
            else
                cheatOn = false;
        }

        public void setEnemies(bool on)
        {
            if (on)
                enemiesOn = true;
            else
                enemiesOn = false;
        }

        private int SizeInTiles(int w)
        {
            return w / Tile.SIZE;
        }
    }
}