using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace SuperScooter
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class Sound //: Microsoft.Xna.Framework.GameComponent
    {
        private Song backgroundMusic;
        bool isPaused;
        bool Effects;

        private SoundEffect JumpSound;
        private SoundEffect EnemySound;
        private SoundEffect DeathSound;
        private SoundEffect WinSound;

        /// <summary>
        /// This is constructor
        /// </summary>
        /// <param name="game">current game state. Usually "this".</param>
        public Sound(Game game)
        {
            backgroundMusic = game.Content.Load<Song>(@"Sound/backMusic2");
            MediaPlayer.Play(backgroundMusic);
            MediaPlayer.IsRepeating = true;
            isPaused = false;

            Effects = true;
            JumpSound = game.Content.Load<SoundEffect>(@"Sound/Jump4");
            EnemySound = game.Content.Load<SoundEffect>(@"Sound/Jump3");
            DeathSound = game.Content.Load<SoundEffect>(@"Sound/Jump2");
            WinSound = game.Content.Load<SoundEffect>(@"Sound/Jump");
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        public void Update()
        {
            //do nothing
        }

        public void FadeOut()
        {
            MediaPlayer.Volume = 1.0f;
        }

        public void PlayJumpSound()
        {
            if (Effects)
                JumpSound.Play();  
        }

        public void PlayEnemySound()
        {
            if(Effects)
                EnemySound.Play();
        }

        public void PlayDeathSound()
        {
            if (Effects)
                DeathSound.Play();
        }

        public void PlayWinSound()
        {
            if (Effects)
                WinSound.Play();
        }

        public void ToggleMusic()
        {
            if (!isPaused)
            {
                MediaPlayer.Pause();
                isPaused = true;
               // Effects = false;
            }
            else
            {
                MediaPlayer.Resume();
                isPaused = false;
               // Effects = true;
            }
        }
        
    }
}